unit Blogs;

interface

uses
  System.Collections, System.ComponentModel,
  System.Data, System.Drawing, System.Web, System.Web.SessionState,
  System.Web.UI, System.Web.UI.WebControls, System.Web.UI.HtmlControls,
  Borland.Eco.ObjectRepresentation,
  Borland.Eco.Handles, WeblogEcoSpace, CoreClassesUnit;

type
  TWebForm1 = class(System.Web.UI.Page)
  {$REGION 'Designer Managed Code'}
  strict private
    procedure InitializeComponent;
                procedure DataGrid1_SelectedIndexChanged(sender: System.Object; 
                        e: System.EventArgs);
                procedure btnNewCategory_Click(sender: System.Object; e: System.EventArgs);
                procedure btnNewPost_Click(sender: System.Object; e: System.EventArgs);
  strict protected  
    rhRoot: Borland.Eco.Handles.ReferenceHandle;
                ehCategories: Borland.Eco.Handles.ExpressionHandle;
                dgCategories: System.Web.UI.WebControls.DataGrid;
                btnNewCategory: System.Web.UI.WebControls.Button;
                ehPosts: Borland.Eco.Handles.ExpressionHandle;
                dgPosts: System.Web.UI.WebControls.DataGrid;
                btnNewPost: System.Web.UI.WebControls.Button;
                lbCategory: System.Web.UI.WebControls.Label;
  {$ENDREGION}
  strict private
    fEcoSpace: TWeblogEcoSpace;
    procedure Page_Load(sender: System.Object; e: System.EventArgs);
    procedure Page_Unload(sender: System.Object; e: System.EventArgs);
  strict protected
    procedure OnInit(e: EventArgs); override;
    function get_EcoSpace: TWeblogEcoSpace;
    procedure DoneWithEcoSpace;
    property EcoSpace: TWeblogEcoSpace read get_EcoSpace;
  public
    procedure DataBind; override;
  strict protected
  {$REGION 'Utility Methods'}
    procedure UpdateDatabase;
    function IdForObject(Obj: IObject): string;
    function ObjectForId(Id: string): IObject;
  {$ENDREGION}
  {$REGION 'DataGrid Events'}
  procedure DataGrid_DeleteCommand(source: TObject;e: System.Web.UI.WebControls.DataGridCommandEventArgs);
  procedure DataGrid_EditCommand(source: TObject;e: System.Web.UI.WebControls.DataGridCommandEventArgs);
  procedure DataGrid_UpdateCommand(source: TObject;e: System.Web.UI.WebControls.DataGridCommandEventArgs);
  procedure DataGrid_CancelCommand(source: TObject;e: System.Web.UI.WebControls.DataGridCommandEventArgs);
  {$ENDREGION}
  end;

implementation

uses
  Borland.Eco.Services, EcoSpaceProvider;

{$REGION 'Designer Managed Code'}
/// <summary>
/// Required method for Designer support -- do not modify
/// the contents of this method with the code editor.
/// </summary>
procedure TWebForm1.InitializeComponent;
begin
  Self.rhRoot := Borland.Eco.Handles.ReferenceHandle.Create;
  Self.ehCategories := Borland.Eco.Handles.ExpressionHandle.Create;
  Self.ehPosts := Borland.Eco.Handles.ExpressionHandle.Create;
  Include(Self.dgCategories.CancelCommand, Self.DataGrid_CancelCommand);
  Include(Self.dgCategories.EditCommand, Self.DataGrid_EditCommand);
  Include(Self.dgCategories.UpdateCommand, Self.DataGrid_UpdateCommand);
  Include(Self.dgCategories.DeleteCommand, Self.DataGrid_DeleteCommand);
  Include(Self.dgCategories.SelectedIndexChanged, Self.DataGrid1_SelectedIndexChanged);
  Include(Self.btnNewCategory.Click, Self.btnNewCategory_Click);
  Include(Self.dgPosts.DeleteCommand, Self.DataGrid_DeleteCommand);
  Include(Self.btnNewPost.Click, Self.btnNewPost_Click);
  // 
  // rhRoot
  // 
  Self.rhRoot.EcoSpaceType := TypeOf(WeblogEcoSpace.TWeblogEcoSpace);
  Self.rhRoot.StaticValueTypeName := 'Category';
  // 
  // ehCategories
  // 
  Self.ehCategories.AddExternalId := True;
  Self.ehCategories.Expression := 'Category.allInstances->orderby(Name)';
  Self.ehCategories.RootHandle := Self.rhRoot;
  // 
  // ehPosts
  // 
  Self.ehPosts.AddExternalId := True;
  Self.ehPosts.Expression := 'self.Posts->orderdescending(Posted)';
  Self.ehPosts.RootHandle := Self.rhRoot;
  Include(Self.Unload, Self.Page_Unload);
  Include(Self.Load, Self.Page_Load);
end;
{$ENDREGION}

procedure TWebForm1.Page_Load(sender: System.Object; e: System.EventArgs);
var
  Id: string;
begin
  EcoSpace.Active := True;
  Id := Request.Params['RootId'];
  if Assigned(Id) and (Id <> '') then
  begin
    rhRoot.SetElement(ObjectForId(Id));
    lbCategory.Text := (rhRoot.Element.AsObject as Category).Name; // BS
    if (rhRoot.Element.AsObject as Category).Posts.Count = 0 then
    begin
      lbCategory.Text := 'No Posts in category ' + lbCategory.Text;
      dgPosts.Visible := False
    end
    else dgPosts.Visible := True
  end
  else // BS - no category selected
  begin
    lbCategory.Text := 'All Posts';
    ehPosts.Expression := 'Post.allInstances->orderdescending(Posted)'
  end;
  if not IsPostBack then
    DataBind;
  // TODO: Put user code to initialize the page here
  btnNewCategory.Visible := User.Identity.IsAuthenticated;
  btnNewPost.Visible := User.Identity.IsAuthenticated;
  dgPosts.Columns[3].Visible := User.Identity.IsAuthenticated
end;

procedure TWebForm1.Page_Unload(sender: System.Object; e: System.EventArgs);
begin
	DoneWithEcoSpace();
end;

procedure TWebForm1.DataBind;
begin
	ElementHandle.EnsureAllBindingLists;
  inherited DataBind;
end;

{$REGION 'Utility Methods'}
procedure TWebForm1.UpdateDatabase;
var
  PersistenceService: IPersistenceService;
  DirtyListService: IDirtyListService;
begin
  PersistenceService := EcoSpace.GetEcoService(typeof(IPersistenceService)) as IPersistenceService;
  DirtyListService := EcoSpace.GetEcoService(typeof(IDirtyListService)) as IDirtyListService;
  PersistenceService.UpdateDatabaseWithList(DirtyListService.AllDirtyObjects());
end;

function TWebForm1.IdForObject(Obj: IObject): string;
var
  IdService: IExternalIdService;
begin
  IdService := EcoSpace.GetEcoService(typeof(IExternalIdService)) as IExternalIdService;
  Result := IdService.IdForObject(Obj);
end;

function TWebForm1.ObjectForId(Id: string): IObject;
var
  IdService: IExternalIdService;
begin
  IdService := EcoSpace.GetEcoService(typeof(IExternalIdService)) as IExternalIdService;
  Result := IdService.ObjectForId(Id);
end;

{$ENDREGION}

{$REGION 'DataGrid Events'}

// The following code is useful if the form contains a System.Web.UI.Datagrid;
// It assumes that the elementhandle acting as a datasource has
// AddExternalId = true, and that datagrid.DataKeyField = "ExternalId"

procedure TWebForm1.DataGrid_DeleteCommand(source: TObject;e: System.Web.UI.WebControls.DataGridCommandEventArgs);
var
  SourceGrid: DataGrid;
  Id: string;
  ToDelete: IObject;
begin
  SourceGrid := source as DataGrid;
  Id := SourceGrid.DataKeys[e.Item.ItemIndex].ToString;
  ToDelete := ObjectForId(Id);
	if Assigned(ToDelete) then
    ToDelete.Delete;
  UpdateDatabase;
  DataBind;
end;

procedure TWebForm1.DataGrid_EditCommand(source: TObject;e: System.Web.UI.WebControls.DataGridCommandEventArgs);
begin
  (source as DataGrid).EditItemIndex := e.Item.ItemIndex;
  DataBind;
end;

procedure TWebForm1.DataGrid_UpdateCommand(source: TObject;e: System.Web.UI.WebControls.DataGridCommandEventArgs);
var
  SourceGrid: DataGrid;
  Id: string;
  Row: TObject;
  Eh: ElementHandle;
  ColIndex: Integer;
  Col: BoundColumn;
begin
  SourceGrid := source as DataGrid;
  Id := SourceGrid.DataKeys[e.Item.ItemIndex].ToString;
  SourceGrid.DataBind(); // To set DataSource
  Eh := SourceGrid.DataSource as ElementHandle;
  Row := eh.RenderElement(ObjectForId(Id));
  for colIndex := 0 to SourceGrid.Columns.Count-1 do
  begin
    Col := BoundColumn(SourceGrid.Columns[colIndex]);
    if Assigned(Col) and (not Col.ReadOnly) then
      eh.SetRenderedElementProperty(row, Col.DataField, (e.Item.Cells[colIndex].Controls[0] as TextBox).Text, true);
  end;
	UpdateDatabase;
	SourceGrid.EditItemIndex := -1;
	DataBind;
end;

procedure TWebForm1.DataGrid_CancelCommand(source: TObject;e: System.Web.UI.WebControls.DataGridCommandEventArgs);
begin
  (source as DataGrid).EditItemIndex := -1;
  DataBind;
end;

procedure TWebForm1.btnNewPost_Click(sender: System.Object; e: System.EventArgs);
var
  NewPost: Post;
begin
  NewPost := Post.Create(EcoSpace);
  NewPost.Author := 'Bob Swart';
  NewPost.Title := 'Title';
  NewPost.Posted := DateTime.Now;
  NewPost.Categories.Add(rhRoot.Element.AsObject as Category);
  UpdateDatabase;
  DataBind;
  lbCategory.Text := (rhRoot.Element.AsObject as Category).Name;
  dgPosts.Visible := True
end;

procedure TWebForm1.btnNewCategory_Click(sender: System.Object; e: System.EventArgs);
var
  NewCategory: Category;
begin
  NewCategory := Category.Create(EcoSpace);
  NewCategory.Name := 'New Category';
  UpdateDatabase;
  DataBind; // update display
end;

procedure TWebForm1.DataGrid1_SelectedIndexChanged(sender: System.Object; e: System.EventArgs);
begin

end;
{$ENDREGION}

procedure TWebForm1.OnInit(e: EventArgs);
begin
  //
  // Required for Designer support
  //
  InitializeComponent;
  inherited OnInit(e);
end;

function TWebForm1.get_EcoSpace: TWeblogEcoSpace;
begin
  if not Assigned(fEcoSpace) then
  begin
    fecoSpace := TEcoSpaceProvider.GetEcoSpace;
		rhRoot.EcoSpace := fEcoSpace;
		// Set additional root handles here and in DoneWithEcoSpace
  end;
  Result := fEcoSpace;
end;

procedure TWebForm1.DoneWithEcoSpace;
begin
  if not Assigned(fEcoSpace) then
    Exit;
  rhRoot.EcoSpace := nil;
  rhRoot.SetElement(nil);
  // Clear all other references to ecoSpace or it's contents here.
	// Otherwise it is not safe to return it to a pool.

  TEcoSpaceProvider.ReturnEcoSpace(fEcoSpace);
  fEcoSpace := nil;
end;

end.

